/****************************************************************************
 Copyright (c) 2017 Chukong Technologies Inc.

 http://www.cocos.com

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated engine source code (the "Software"), a limited,
  worldwide, royalty-free, non-assignable, revocable and  non-exclusive license
 to use Cocos Creator solely to develop games on your target platforms. You shall
  not use Cocos Creator software for developing other software or tools that's
  used for developing games. You are not granted to publish, distribute,
  sublicense, and/or sell copies of Cocos Creator.

 The software or tools in this License Agreement are licensed, not sold.
 Chukong Aipu reserves all rights not expressly granted to you.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 ****************************************************************************/
var fs = qg.getFileSystemManager ? qg.getFileSystemManager() : null;

var ID = 'MiDownloader';

var non_text_format = [
    'js', 'png', 'jpg', 'bmp', 'jpeg', 'gif', 'ico', 'tiff', 'webp', 'image', 'mp3', 'ogg', 'wav', 'm4a', 'font', 'eot', 'ttf', 'woff', 'svg', 'ttc'
];

const REGEX = /^\w+:\/\/.*/;

var MiDownloader = window.MiDownloader = function () {
    this.id = ID;
    this.async = true;
    this.pipeline = null;
    this.REMOTE_SERVER_ROOT = '';
    this.SUBCONTEXT_ROOT = '';
};

MiDownloader.ID = ID;

var miDownloader = window.miDownloader = new MiDownloader();

var RES_VERSIONS = "";
var LOCAL_FILE_PATH = qg.env.USER_DATA_PATH; // "internal://files";

MiDownloader.prototype.init = function () {
    RES_VERSIONS = (function(){
        let serResPath = miDownloader.REMOTE_SERVER_ROOT;
        if (serResPath && serResPath.length) {
            let strs = serResPath.split("/");
            if (strs.length) {
                return strs[strs.length-1] + "/";
            }
        }
        return "";
    })();

    (function() {
        var verPath = LOCAL_FILE_PATH + "/" + RES_VERSIONS + "ver.json";
        var resetVersion = function () {
            console.log("版本更新: " + RES_VERSIONS);
            fs.rmdir({
                uri: LOCAL_FILE_PATH,
                complete: function () {
                    fs.writeFile({
                        filePath: verPath,
                        data: "1",
                        fail: function() {
                            console.log("版本更新: 失败");
                        },
                        success: function() {
                            console.log("版本更新: 成功");
                        }
                    });
                }
            });
        };
        fs.access({
            path: verPath,
            fail: function() {
                resetVersion();
            },
            success: function() {
                console.log("当前版本: " + RES_VERSIONS);
            }
        });
    })();
};

MiDownloader.prototype.handle = function (item, callback) {
    if (item.type === 'js') {
        return null;
    }
    if (item.type === 'uuid') {
        var result = cc.Pipeline.Downloader.PackDownloader.load(item, callback);
        // handled by PackDownloader
        if (result !== undefined) {
            // null result
            if (!!result) {
                return result;
            }
            else {
                return;
            }
        }
    }

    var filePath = item.url;
    // filter protocol url (E.g: https:// or http:// or ftp://)
    if (!miDownloader.REMOTE_SERVER_ROOT || miDownloader.REMOTE_SERVER_ROOT == "" || REGEX.test(filePath)) {
        callback(null, null);
        return;
    }
    var localPath = LOCAL_FILE_PATH + "/" + RES_VERSIONS + filePath;
    fs.access({
        path: localPath,
        success: function() {
            item.url = localPath;
            item.rawUrl = localPath;
            readFromLocal(item, callback);
        },
        fail: function() {
            downloadRemoteFile(item, callback);
        },
    });
};

function nextPipe (item, callback) {
    var queue = cc.LoadingItems.getQueue(item);
    queue.addListener(item.id, function (item) {
        if (item.error) {
            fs.unlink({
                filePath: item.url,
                // success: function () {
                //     cc.log('Read file failed, removed local file ' + url + ' successfully!');
                // }
            });
        }
    });
    callback(null, null);
};

function readText (item, callback) {
    var url = item.url;
    fs.readFile({
        filePath: url,
        encoding: 'utf8',
        success: function (res) {
            item.states[cc.loader.downloader.id] = cc.Pipeline.ItemState.COMPLETE;
            callback(null, res.data);
        },
        fail: function (res) {
            // console.warn('Read file failed: ' + url);
            fs.unlink({
                filePath: url,
                // success: function () {
                //     cc.log('Read file failed, removed local file ' + url + ' successfully!');
                // }
            });
            callback({
                status: 0,
                errorMessage: res && res.errMsg ? res.errMsg : "Read text file failed: " + url
            });
        }
    });
};

function readFromLocal (item, callback) {
    if (item.type && non_text_format.indexOf(item.type) !== -1) {
        nextPipe(item, callback);
    } else {
        readText(item, callback);
    }
};

function downloadRemoteFile (item, callback, count) {
    var relatUrl = item.url;
    // filter protocol url (E.g: https:// or http:// or ftp://)
    var isnet = REGEX.test(relatUrl);
    if (isnet) {
        callback(null, null);
        return;
    }
    var remoteUrl = miDownloader.REMOTE_SERVER_ROOT + '/' + relatUrl;
    qg.downloadFile({
        url: remoteUrl,
        success: function (res) {
            if (res.statusCode === 404) {
                // console.warn("Download file failed: " + remoteUrl);
                callback({
                    status: 0,
                    errorMessage: res && res.errMsg ? res.errMsg : "Download file failed: " + remoteUrl
                });
            }
            else if (res.tempFilePath) {
                var tempPath = res.tempFilePath;
                var localPath = LOCAL_FILE_PATH + "/" + RES_VERSIONS + relatUrl;
                fs.saveFile({
                    tempFilePath: tempPath,
                    filePath: localPath,
                    success: function (res) {
                        item.url = localPath;
                        readFromLocal(item, callback);
                    },
                    fail: function (res) {
                        // cc.log('save file failed:' + res.errMsg);
                        item.url = tempPath;
                        readFromLocal(item, callback);
                    }
                });
            }
        },
        fail: function (res) {
            if (count && count > 10) {
                callback({
                    status: 0,
                    errorMessage: res && res.errMsg ? res.errMsg : "Download file failed: " + remoteUrl
                }, null);
                return;
            }
            setTimeout(function () {
                downloadRemoteFile(item, callback, count ? count + 1 : 1);
            }, 10000);
        }
    });
};
